## Description: Wrapper script for codebook generation. Call `Rscript codebook_generator.R -h` for more details
##
## Author: Rahul Somani
## Date Last Modified: June-7-2020

library(optparse)
library(rmarkdown)

option_list <- list(
    make_option(c('-i', '--path_notebook'), type='character',
                default='./codebook_doc.Rmd',
                help='Path to the Rmarkdown notebook that generates the codebook'),
    make_option(c('-p', '--path_codebook_preprocessor'), type='character',
                default="./codebook_preprocess.R",
                help='Path to the script that preprocess the data used to generate
                      the codebook.
                      Default = "./codebook_preprocess.R'),
    make_option(c('-o', '--path_output'), type='character',
                default="../../../DOC/CODEBOOK",
                help='Path where the output should be generated.
                      Default = "../../../DOC/CODEBOOK"'),
    make_option(c('-f', '--output_fname'), type='character',
                default="codebook.pdf",
                help='Name of the output file. Must include `.pdf` extension.
                      Default = "codebook.pdf"'),
    make_option(c('-d', '--path_data'), type='character',
                default="",
                help='Path to the scraped data')
)

program_summary <- "This script is a wrapper to call the Rmarkdown file that generates the codebook. It takes as inputs the path to the scraped-data, Rmarkdown file, name of the output file, and the path where the output file must be generated. It saves the intermediary .tex files and figures generated as well.

Usage Example
=============

Rscript codebook_generator.R \
    --path_notebook  ./codebook_doc.Rmd
    --path_data      ~/scraped-data-directory
    --path_output    ../../../DOC/CODEBOOK
    --output_fname   codebook.pdf
"

opt <- parse_args(OptionParser(option_list=option_list, usage=program_summary))

path <- opt$path_data
source(opt$path_codebook_preprocessor)

render(input             = opt$path_notebook,
       output_file       = opt$output_fname,
       output_dir        = opt$path_output,
       intermediates_dir = opt$path_output)

