suppressPackageStartupMessages(library(dplyr))
suppressPackageStartupMessages(library(glue))
suppressPackageStartupMessages(library(purrr))
suppressPackageStartupMessages(library(stringr))
suppressPackageStartupMessages(library(readr))
library(optparse)

option_list <- list(
    make_option(c('-p', '--path_data'), type='character',
                default='~/Desktop/other/IGIDR/NCLT/',
                help='Path where all the NCLT data is stored'),
    make_option(c('-i', '--input_cpca_file'), type='character',
                default='gen_full_cpcas_output.txt',
                help='Path to the file with CP/CA numbers of full cases
                (generated by `01_generate_full_cpca_numbers.R`)'),
    make_option(c('-o', '--path_output'), type='character',
                default=getwd(),
                help='Path where the output must be saved.
                Default = current directory'),
    make_option(c('-f', '--output_filename'), type='character',
                default='frg_nclt_cauelist_dataset',
                help='Name of the output file of the dataset with full cases.
                Do not include extension.
                Default = "frg_nclt_cauelist_dataset"')
)

get_all_data <- function(path="~/Desktop/NCLT") {
    list.files(path, recursive=TRUE, full.names=TRUE) %>%
    map(., function(x) str_extract(x, ".*data/.*csv")) %>% unlist %>%
    discard(is.na) %>%
    map_df(read_csv, col_types=cols(Section = col_character()))
}

program_summary <- "This script takes as input a text file generated by `01_generate_full_cpca_numbers.R` and the path to the scraped NCLT data, and generates a csv file of the unprocessed NCLT Cause-Lists dataset with all the hearings of cases with these CP/CA numbers.


Usage Example
=============

Rscript gen_full_cpcas_dataset.R \
    --path_data       ~/NCLT-SCRAPED-DATA \
    --input_cpca_file ~/full_cpcas.txt \
    --path_output     ~/ \
    --output_filename full_cpcas_dataset
"

opt  <- parse_args(OptionParser(option_list=option_list, usage=program_summary))
path <- opt$path_data
path_cpca  <- opt$input_cpca_file
path_out   <- opt$path_output
out_fname  <- opt$output_filename
out_file   <- glue("{path_out}/{out_fname}.csv")

full_cpcas <- path_cpca %>% read_delim("\n", col_names=F)
df         <- get_all_data(path=path)

cases_full_raw <-
    filter(df, `CP/CA No` %in% full_cpcas$X1) %>%
    mutate(Source_File_CPCA_Nos = path_cpca)

write_csv(cases_full_raw, out_file)
