## Description: Takes as input the `benches_courts_names.pkl` file from
##   the first script and validates that they are valid. If they are, it
##   changes the file-name to `benches_courts-VALIDATED.pkl`. If the
##   validation check fails, run the first script again.
## Run `python _02_verify_bench_courts.py -h` to see info on the arguments you can pass in.
## Author: Rahul Somani
## Date Last Revisited: 06-June-2020

from imports import *
from utils import *
from html_parsing import *

def validate(bench_courts:dict,
             browser:Firefox=0,
             browser_headless:bool=True,
             url:str = 'https://nclt.gov.in/exposed-status-of-cause-list-page',
             verbose:bool = True,
             sleep_select:float = 2.0,
             sleep_pageload:float = 5.0
            ) -> bool:
    # if browser doesn't exist, create one
    try: browser.current_url
    except AttributeError:
        options = FirefoxOptions()
        options.headless = browser_headless
        browser = Firefox(options=options)

    # Try opening every bench-court combination.
    # If the page opens, the combos are correct.
    for bench,courts in bench_courts.items():
        for court in courts:
            try:
                browser = open_cause_list(browser, bench,
                                          court, pd.Timestamp('20200103'),
                                          sleep_select=sleep_select,
                                          sleep_pageload=sleep_pageload,
                                          verbose=verbose)
            except NoSuchElementException:
                print('''
                            !!! VALIDATION FAILED !!!
                            =========================
                Bench-court combinations were NoSuchElementExceptiond incorrectly.
                Either retry running the validation check, or
                check the bench-court combinations manually.
                ''')
                browser.quit()
                return
    print('''
        !!! VALIDATION CHECK PASSED !!!
        ===============================
    All grabbed bench-court combos are legit.
    ''')
    browser.quit()
    return True

@call_parse
def main(load_path:Param('Path to the saved `pkl` (dict) file', str)='benches_courts_names.pkl',
         headless_browser:Param('Run virtual browser in headless mode', bool_arg) = True,
         url:Param('URL of the NCLT Cause Lists (HTML) Webpage',
         str) = 'https://nclt.gov.in/exposed-status-of-cause-list-page',
         verbose:Param('Verbose output?', bool_arg) = True,
         sleep_select:Param('How long to sleep after entering the court', float) = 2.0,
         sleep_pageload:Param('How long to wait after clicking the `apply` button?', float) = 4.0):

         load_path = Path(load_path)
         bench_courts = load_pickle(load_path=load_path)
         valid = validate(bench_courts, browser_headless=headless_browser, verbose=verbose,
                          sleep_select=sleep_select, sleep_pageload=sleep_pageload)

         if valid:
             print(f"Renaming {load_path} to {load_path/'benches_courts-VALIDATED'}.pkl")
             load_path.rename('benches_courts-VALIDATED.pkl')
