## Description:  This script uses the `bench_courts_url.csv` file
##   generated by the previous script to scrape data from the
##   website. Takes a start + end date as an argument, and a path to save
##   the data to.
##
## Run `python _04_get_cause_lists.py -h` to see info on the arguments you can pass in.
##
## Author: Rahul Somani
## Date Last Revisited: 06-June-2020

from imports import *
from html_parsing import *
from utils import *

@call_parse
def main(data_dir:Param('Directory where the data gets stored. If it does not exist, it will be created.',
                        str) = '~/Desktop/NCLT-NEW',
         geckodriver_path:Param('Path to the (firefox) geckodriver. If you do not have this, download the latest version from https://github.com/mozilla/geckodriver/releases, unzip and store it in "/usr/local/bin/"',
                                str) = '/usr/local/bin/geckodriver',
         bench_court_urls_csv:Param('Path to the csv generated using `_03_get_bench_courts_urls.py`',
                                    str) = 'bench_court_urls.csv',
         start_date:Param('Date to start scraping, in YYYYMMDD format. Must be >= 20190101', str) = '20190101',
         end_date:  Param('Date to end scraping, in YYYYMMDD format. If none, defaults to today', str, required=False) = None,
         dates:     Param('Must be in the format YYYYMMDD. Enter 1 or more dates to download the cause list for, OR enter the path to a csv file containing a column full of dates in YYYY-MM-DD format', str, nargs='*', required=False) = None,
         bench:     Param('Scrape data for only this bench. Enclose in quotes.', str, nargs=1, required=False) = None,
         courts:    Param("Use this argument only if you're using the `bench` argument. Enter the specific courts you'd like to scrape data for. If unused, data for all courts from the `bench` will be scraped",
                          str, nargs='*', required=False) = None,
         headless_browser:Param('Run browser in headless mode?', bool_arg) = True,
         sleep_pageload:Param('How long to wait after loading the date-specific url', int) = 4,
         sleep_before_next:Param('How long to sleep before trying the next url', int) = 15):

         try:
             dates = pd.read_csv(dates)
             dates = [str(date[0]) for date in dates.values]
             dates = get_dates(start_date, end_date, dates, format="%Y-%m-%d")
         except ValueError:
             dates = get_dates(start_date, end_date, dates)

         base_dir = Path(data_dir)
         options  = FirefoxOptions()
         options.headless = headless_browser

         if not base_dir.exists(): Path.mkdir(base_dir)

         bc_df = pd.read_csv(bench_court_urls_csv)
         df    = get_bench_court_df(bc_df, bench, courts)
         benches, courts, target_ids, entity_ids, _ = df2lists(df)

         for bench, court, target_id, entity_id in zip(benches, courts, target_ids, entity_ids):
             print(f'Bench ==> {bench}')
             print(f'Court ==> {court}')
             print('========================================')

             no_data = []
             verify_dates = []

             bc_fname = f"{bench.lower().replace(' ','-')}_{court.lower()}"
             no_data_fname = f"no_data_{bc_fname}"
             verify_dates_fname = f"verify_dates_{bc_fname}"
             data_dir, log_dir = makedirs(base_dir, bc_fname)

             for date in dates:
                 day   = date.day
                 month = date.month
                 year  = date.year
                 date_str = date.strftime('%Y-%m-%d')
                 fname = f"{date_str}_{bc_fname}"

                 url = f'https://nclt.gov.in/status-of-cause-list?field_date_value[value][day]={day}&field_date_value[value][month]={month}&field_date_value[value][year]={year}&field_bench_target_id={target_id}&field_bench_court_target_id_entityreference_filter={entity_id}'
                 browser = Firefox(options=options, executable_path=geckodriver_path)
                 browser.get(url)
                 time.sleep(sleep_pageload)
                 soup = None
                 soup = BeautifulSoup(browser.page_source, features='lxml')
                 _ = soup.findAll(attrs={'class': re.compile('view-status-of-cause-list view-id-status_of_cause_list')})
                 empty = 'currently content not available' in str(_).lower()

                 if empty:
                     no_data.append(date)
                     print(f'No data for {date_str}')
                 else:
                     table = None
                     table = get_data(soup, bench, court, date)
                     if table is None:
                         verify_dates.append(date)
                         continue
                         print(f'Data might exist for {date_str} but was not extracted. Verify this.')
                     else:
                         table.to_csv(f'{data_dir/fname}.csv', index = False)
                         print(f'Succesfully extracted data for {date_str}')

                 no_data_csv = pd.DataFrame(no_data, columns = ['date'])
                 verify_dates_csv = pd.DataFrame(verify_dates, columns = ['date'])
                 browser.quit()
                 time.sleep(sleep_before_next)

             new_no_data       = pd.DataFrame(no_data, columns = ['date'])
             new_verify_dates  = pd.DataFrame(verify_dates, columns = ['date'])

             if os.path.exists(no_data_fname):
                 prev = pd.read_csv(no_data_fname)
                 #print('(OLD) no_data')
                 #print(prev)
                 no_data_csv = pd.concat((prev, new_no_data), ignore_index = True)
                 no_data_csv['date'] = pd.to_datetime(no_data_csv.date)
                 no_data_csv.sort_values(by=['date'], inplace=True)
                 no_data_csv.drop_duplicates(inplace=True)
                 no_data_csv.reset_index(drop=True, inplace=True)
                 #print('Merged No Data')
                 print(no_data_csv)
                 print(log_dir/no_data_fname)
                 #os.remove(log_dir/no_data_fname)
                 no_data_csv.to_csv(f'{log_dir/no_data_fname}.csv', index=False)
             else:
                 new_no_data.to_csv(f'{log_dir/no_data_fname}.csv', index=False)

             if os.path.exists(verify_dates_fname):
                 prev = pd.read_csv(verify_dates_fname)
                 verify_dates_csv = pd.concat((prev, new_verify_dates), ignore_index = True)
                 verify_dates_csv['date'] = pd.to_datetime(verify_dates_csv.date)
                 verify_dates_csv.sort_values(by=['date'], inplace=True)
                 verify_dates_csv.drop_duplicates(inplace=True)
                 verify_dates_csv.reset_index(drop=True, inplace=True)
                 #os.remove(log_dir/verify_dates_fname)
                 verify_dates_csv.to_csv(f'{log_dir/verify_dates_fname}.csv', index=False)
             else:
                 new_verify_dates.to_csv(f'{log_dir/verify_dates_fname}.csv', index=False)

             print()

         browser.quit()
