from imports import *
from utils import *

def open_cause_list(browser:Firefox,
                    bench:str,
                    court:str,
                    date:pd.Timestamp,
                    url:str      = 'https://nclt.gov.in/exposed-status-of-cause-list-page',
                    id_day:str   = 'edit-field-date-value-value-day',
                    id_month:str = 'edit-field-date-value-value-month',
                    id_year:str  = 'edit-field-date-value-value-year',
                    id_bench:str = 'edit-field-bench-target-id',
                    id_court:str = 'edit-field-bench-court-target-id-entityreference-filter',
                    sleep_select:float = 2.5,
                    sleep_pageload:float = 5.0,
                    verbose:bool = True
                    ) -> Firefox:
    'For a given `bench`, `court` and `date`, open the cause list page'

    if verbose: print(f'Opening {bench} -- {court}')

    if url is not None:
        if verbose: print(f'Loading {url}')
        browser.get(url)
        if verbose: print(f'Loaded url')
        if verbose: print('--------------------------------------')

    # store elements of the date
    year  = date.year
    month = date.month
    day   = date.day

    # find all the relevant elements
    select_day   = Select(browser.find_element_by_id(id_day))
    select_month = Select(browser.find_element_by_id(id_month))
    select_year  = Select(browser.find_element_by_id(id_year))
    select_bench = Select(browser.find_element_by_id(id_bench))
    select_court = Select(browser.find_element_by_id(id_court))

    # select all the relevant fields
    select_day.select_by_index(day)
    select_month.select_by_index(month)
    select_year.select_by_visible_text(str(year))
    select_bench.select_by_visible_text(bench)

    # let the website load the court options. wait for 2-4 seconds
    if verbose: print(f'Entered the court name. Waiting for {sleep_select} seconds for court options to load')
    time.sleep(sleep_select)
    select_court.select_by_visible_text(court)

    # click the button to show the cause list
    try:
        browser.find_element_by_id('front-status-of-cause-list-button').click()
    except NoSuchElementException:
        browser.find_element_by_id('front-customised-cause-list-button').click()

    if verbose: print(f'Entered, clicked all inputs. Waiting for {sleep_pageload} seconds for the page to load')
    time.sleep(sleep_pageload) # wait until page loads...
    if verbose: print('==========================================================================')
    return browser

def extract_target_id(url:str) -> str:
    x = re.findall('field_bench_target_id=\d.*&', url)[0]
    return x.replace('field_bench_target_id=', '').replace('&', '')

def extract_entity_reference_id(url:str) -> str:
    x = re.findall('entityreference_filter=\d.*', url)[0]
    return x.replace('entityreference_filter=','')

def get_data(soup:BeautifulSoup,
             bench:str, court:str, date:pd.Timestamp) -> pd.DataFrame:

    tables_html = soup.find_all('table', attrs={'class':'views-table cols-7'})

    if(tables_html == []): return None
    else:
        tables      = pd.read_html(str(tables_html))
        corams      = []
        for coram in soup.find_all('ol', attrs={'class':'coram-sub-menu'}):
            corams.append([member.getText() for member in coram.findAll('li')])

        for coram, table in zip(corams, tables):
            table['Date']  = date.strftime("%Y-%m-%d")
            table['Coram'] = '; '.join(coram)
            table['Bench'] = bench
            table['Court'] = court

        colnames = ['S.No', 'Date', 'Bench', 'Court', 'CP/CA No', 'Case Purpose', 'Section',
                    'Name of Parties', 'Name of the Legal Practitioner', 'Remarks', 'Coram']

        return pd.concat(tables)[colnames]
