from imports import *

def dict2df(bench_court_dict:dict) -> pd.DataFrame:
    bench_, court_ = [],[]
    for bench,courts in bench_court_dict.items():
        for court in courts:
            bench_.append(bench)
            court_.append(court)
    return pd.DataFrame({
        'BENCH': bench_,
        'COURT': court_
    })

def save_pickle(any_variable, save_path:Union[str,Path]):
    with open(save_path, 'wb') as f:
        pickle.dump(any_variable, f)

def load_pickle(load_path:Union[str,Path]):
    assert Path(load_path).exists(), 'invalid path'
    with open(load_path, 'rb') as f:
        return pickle.load(f)

def get_dates(start=None, end=None, dates=None, format="%Y%m%d"):
    if dates is None:
        if start is None and end is None: start=end=datetime.today()
        if start is not None and end is None: end=datetime.today()
        return pd.date_range(start, end, freq='B')
    else:
        assert(start is None and end is None), 'Enter either (start and/or end date) or a list of dates'
        return [datetime.strptime(date, format).date() for date in dates]

def get_bench_court_df(bc_df:pd.DataFrame, # df with target_it, entity_id, url
                       benches:str,
                       courts:str) -> pd.DataFrame:
    if benches is not None:
        if courts is not None:
            assert(len(benches) == 1), 'If you want specific courts from a bench, only enter the name of one bench'
            return bc_df[(bc_df.bench.isin(benches)) & (bc_df.court.isin(courts))]
        else:
            return bc_df[bc_df.bench.isin(benches)]
    else:
        return bc_df

def df2lists(df:pd.DataFrame) -> Collection[list]:
    benches = df.bench.tolist()
    courts  = df.court.tolist()
    target_ids  = df.target_id.tolist()
    entity_ids  = df.entity_reference_id.tolist()
    sample_urls = df.sample_url.tolist()
    return (benches, courts, target_ids, entity_ids, sample_urls)

def makedirs(base_dir:Path, bc_fname:str) -> Tuple:
    if not (base_dir/bc_fname).exists(): Path.mkdir(base_dir/bc_fname)
    data_dir = base_dir/bc_fname/'data'
    log_dir  = base_dir/bc_fname/'logs'
    if not data_dir.exists(): Path.mkdir(data_dir)
    if not log_dir.exists():  Path.mkdir(log_dir)
    return (data_dir, log_dir)
